package eu.mhsl.minecraft.pixelblocks.pixelblock;

import eu.mhsl.minecraft.pixelblocks.Main;
import eu.mhsl.minecraft.pixelblocks.utils.ListUtil;
import org.bukkit.Location;
import org.bukkit.NamespacedKey;
import org.bukkit.entity.BlockDisplay;
import org.bukkit.entity.Entity;
import org.bukkit.entity.EntityType;
import org.bukkit.persistence.PersistentDataType;
import org.bukkit.util.Transformation;
import org.jetbrains.annotations.NotNull;

import java.util.List;
import java.util.Objects;

public class Pixels {
    private static final NamespacedKey pixelOfTag = new NamespacedKey(Main.plugin(), "pixel_of");

    private final @NotNull PixelBlock parentBlock;

    public Pixels(@NotNull PixelBlock parentBlock) {
        this.parentBlock = parentBlock;
    }

    public void spawn() {
        ListUtil.splitListInParts(10, this.parentBlock.getPixelData())
            .forEach(pixels -> parentBlock.getBlockTaskChain()
                .delay(1)
                .sync(() -> pixels.forEach(this::spawnPixel))
                .execute());
    }

    private void spawnPixel(PixelBlockWorld.PixelData pixelData) {
        Location pixelBlockLocation = this.parentBlock.getPixelBlockLocation();
        Location pixelLocation = pixelBlockLocation.add(pixelData.relativeLocation().multiply(pixelData.scale()));
        BlockDisplay entity = (BlockDisplay) pixelBlockLocation.getWorld().spawnEntity(pixelLocation, EntityType.BLOCK_DISPLAY);

        entity.setBlock(pixelData.block());
        Transformation transform = entity.getTransformation();
        transform.getScale().set(pixelData.scale());
        entity.setTransformation(transform);
        entity.getPersistentDataContainer().set(pixelOfTag, PersistentDataType.STRING, this.parentBlock.getBlockUUID().toString());
    }

    public void destroy() {
        List<BlockDisplay> entities = parentBlock.getPixelBlockLocation().getNearbyEntitiesByType(BlockDisplay.class, 1)
            .stream()
            .filter(blockDisplay -> blockDisplay.getPersistentDataContainer().has(pixelOfTag))
            .filter(blockDisplay -> Objects.equals(
                blockDisplay.getPersistentDataContainer().get(pixelOfTag, PersistentDataType.STRING),
                parentBlock.getBlockUUID().toString()
            ))
            .toList();

        ListUtil.splitListInParts(10, entities)
            .forEach(pixels -> parentBlock.getBlockTaskChain()
                .delay(1)
                .sync(() -> pixels.forEach(Entity::remove))
                .execute());
    }
}
